@@ -1,9 +1,20 @@
-define(["nbextensions/vpython_libraries/jquery-ui.custom.min",
-        "nbextensions/vpython_libraries/glow.min"], function() {
+define(["nbextensions/vpython_libraries/plotly.min",
+        "nbextensions/vpython_libraries/glow.min",
+        "nbextensions/vpython_libraries/jquery-ui.custom.min"], function(Plotly) {
 
-comm = IPython.notebook.kernel.comm_manager.new_comm('glow')
-comm.on_msg(onmessage)
-console.log("comm created for glow target", comm)
+var comm
+
+window.Plotly = Plotly
+
+IPython.notebook.kernel.comm_manager.register_target('glow',
+    function(commChannel, msg) {
+        // commChannel is the frontend comm instance
+        // msg is the comm_open message, which can carry data
+        comm = commChannel
+        // Register handlers for later messages:
+        comm.on_msg(onmessage);
+        comm.on_close(function(msg) {console.log("glow comm channel closed")});
+    });
 
 //var datadir = '../vpython_data/'
 var datadir = requirejs.s.contexts._.config.paths.nbextensions + '/vpython_data/'
@@ -48,7 +59,7 @@ function send() { // periodically send events and update_canvas and request obje
 	var update = update_canvas()
 	if (update !== null) events = events.concat(update)
 	if (events.length === 0) events = [{event:'update_canvas', 'trigger':1}]
-	comm.send( events )
+	if (comm) comm.send( events )
 	events = []
 }
 
@@ -72,6 +83,7 @@ var lastpos = vec(0,0,0)
 var lastray = vec(0,0,0)
 var lastforward = vec(0,0,0)
 var lastup = vec(0,0,0)
+var lastcenter = vec(0,0,0)
 var lastrange = 1
 var lastautoscale = true
 var lastsliders = {}
@@ -100,9 +112,20 @@ function update_canvas() { // mouse location and other stuff
 			}
 			lastforward = forward
 			var up = cvs.up
-			if (!up.equals(lastup)) {evt.up = [up.x,up.y,up.z]; dosend=true}
+			if (!up.equals(lastup)) {
+                evt.up = [up.x,up.y,up.z]
+                dosend=true
+            }
 			lastup = up
 		}
+        if (cvs.userpan) {
+            var center = cvs.center
+            if (!center.equals(lastcenter)) {
+                evt.center = [center.x,center.y,center.z]
+                dosend=true
+            }
+            lastcenter = center
+        }
 		if (cvs.userzoom) {
 			var range = cvs.range
 			if (range !== lastrange) {evt.range=range; dosend=true}
@@ -266,7 +289,7 @@ function control_handler(obj) {  // button, menu, slider, radio, checkbox
 	events.push(evt)
 }
 
-// attrs are X in {'a': '23X....'}
+// attrs are X in {'a': '23X....'} avaiable: none
 var attrs = {'a':'pos', 'b':'up', 'c':'color', 'd':'trail_color', // don't use single and double quotes; available: comma, but maybe that would cause trouble
          'e':'ambient', 'f':'axis', 'g':'size', 'h':'origin', 'i':'textcolor',
          'j':'direction', 'k':'linecolor', 'l':'bumpaxis', 'm':'dot_color',
@@ -285,7 +308,7 @@ var attrs = {'a':'pos', 'b':'up', 'c':'color', 'd':'trail_color', // don't use s
          
          // text attributes: 
          '$':'text', '%':'align', '^':'caption',
-         '-':'title_align', '&':'title', '*':'xtitle', '(':'ytitle',
+         '-':'fast','&':'title', '*':'xtitle', '(':'ytitle',
          
          // Miscellany:
          ')':'lights', '_':'objects', '=':'bind',
@@ -297,7 +320,10 @@ var attrs = {'a':'pos', 'b':'up', 'c':'color', 'd':'trail_color', // don't use s
 var attrsb = {'a':'userzoom', 'b':'userspin', 'c':'range', 'd':'autoscale', 'e':'fov',
               'f':'normal', 'g':'data', 'h':'checked', 'i':'disabled', 'j':'selected',
               'k':'vertical', 'l':'min', 'm':'max', 'n':'step', 'o':'value',
-              'p':'left', 'q':'right', 'r':'top', 's':'bottom', 't':'_cloneid'}
+              'p':'left', 'q':'right', 'r':'top', 's':'bottom', 't':'_cloneid',
+              'u':'logx', 'v':'logy', 'w':'dot', 'x':'dot_radius', 
+              'y':'markers', 'z':'legend', 'A':'label','B':'delta', 'C':'marker_color',
+              'D':'size_units', 'E':'userpan'}
 
 // methods are X in {'m': '23X....'}
 var methods = {'a':'select', 'b':'pos', 'c':'start', 'd':'stop', 'f':'clear', // unused eghijklmnopvxyzCDFAB
@@ -306,13 +332,13 @@ var methods = {'a':'select', 'b':'pos', 'c':'start', 'd':'stop', 'f':'clear', //
                'G':'bind', 'H':'unbind', 'I':'waitfor', 'J':'pause', 'K':'pick', 'L':'GSprint',
 		       'M':'delete'}
          
-var vecattrs = ['pos', 'up', 'color', 'trail_color', 'axis', 'size', 'origin', 'textcolor',
-                'direction', 'linecolor', 'bumpaxis', 'dot_color', 'ambient', 'add_to_trail',
-                '_attach_arrow',
-                'foreground', 'background', 'ray', 'ambient', 'center', 'forward', 'normal']
+var vecattrs = ['pos', 'up', 'color', 'trail_color', 'axis', 'size', 'origin', '_attach_arrow',
+                'direction', 'linecolor', 'bumpaxis', 'dot_color', 'ambient', 'add_to_trail', 'textcolor',
+                'foreground', 'background', 'ray', 'ambient', 'center', 'forward', 'normal',
+                'marker_color']
                 
 var textattrs = ['text', 'align', 'caption', 'title', 'title_align', 'xtitle', 'ytitle', 'selected',
-                 'append_to_caption', 'append_to_title', 'bind', 'unbind', 'pause', 'GSprint']
+                 'label', 'append_to_caption', 'append_to_title', 'bind', 'unbind', 'pause', 'GSprint']
 
 // patt gets idx and attr code; vpatt gets x,y,z of a vector            
 var patt = /(\d+)(.)(.*)/
@@ -330,7 +356,7 @@ function decode(data) {
 	
 	if ('attrs' in data) {
 		var c = data['attrs']
-		for (i in c) { // step through the encoded attributes and methods
+		for (i=0; i<c.length; i++) { // step through the encoded attributes and methods
 			var d = c[i]
 			// constructor or appendcmd not currently compressed
 			var whichlist = d[0] // 'a' or 'b' or 'm'
@@ -429,7 +455,7 @@ function handler(data) {
 function handle_cmds(dcmds) {
     "use strict";
 	//console.log('CMDS')
-	for (var icmds in dcmds) { // constructors, and special operations such as curve.modify
+	for (var icmds=0; icmds<dcmds.length; icmds++) { // constructors, and special operations such as curve.modify
 		var cmd = dcmds[icmds]
 		var obj = cmd.cmd
 		var idx = cmd.idx
@@ -446,7 +472,7 @@ function handle_cmds(dcmds) {
 		//assembling cfg
 		var vlst = ['pos', 'color', 'size', 'axis', 'up', 'direction', 'center', 'forward', 'foreground',
 				 'background', 'ambient', 'linecolor', 'dot_color', 'trail_color', 'textcolor', 'attrval',
-				 'origin', 'normal', 'bumpaxis','texpos', 'start_face_color', 'end_face_color']
+				 'origin', 'normal', 'bumpaxis','texpos', 'start_face_color', 'end_face_color', 'marker_color']
 		if ((obj != 'gcurve') && ( obj != 'gdots' ) ) vlst.push( 'size' )
 		var cfg = {}
 		var objects = []
@@ -488,7 +514,6 @@ function handle_cmds(dcmds) {
 				cfg[attr] = val
 			} else {
 				cfg[attr] = val
-				// console.log(attr.attr, attr.value)
 			}
 		}
 		if (!construct) { // commands such as "center" (for a canvas)
@@ -525,24 +550,25 @@ function handle_cmds(dcmds) {
 		// creating the objects
 		cfg.idx = idx // reinsert idx, having looped thru all other attributes
 		switch (obj) {
-			case 'box':       {glowObjs[idx] = box(cfg); break}
-			case 'sphere':    {glowObjs[idx] = sphere(cfg); break}
-			case 'arrow':     {glowObjs[idx] = arrow(cfg); break}
-			case 'cone':      {glowObjs[idx] = cone(cfg); break}
-			case 'cylinder':  {glowObjs[idx] = cylinder(cfg); break}
-			case 'helix':     {glowObjs[idx] = helix(cfg); break}
-			case 'pyramid':   {glowObjs[idx] = pyramid(cfg); break}
-			case 'ring':      {glowObjs[idx] = ring(cfg); break}
-			case 'curve':     {glowObjs[idx] = curve(cfg); break}
-			case 'points':    {glowObjs[idx] = points(cfg); break}
-			case 'vertex':    {glowObjs[idx] = vertex(cfg); break}
-			case 'triangle':  {glowObjs[idx] = triangle(cfg); break}
-			case 'quad':      {glowObjs[idx] = quad(cfg); break}
-			case 'label':     {glowObjs[idx] = label(cfg); break}
-			case 'ellipsoid': {glowObjs[idx] = sphere(cfg); break}
+			case 'box':           {glowObjs[idx] = box(cfg); break}
+			case 'sphere':        {glowObjs[idx] = sphere(cfg); break}
+			case 'simple_sphere': {glowObjs[idx] = simple_sphere(cfg); break}
+			case 'arrow':         {glowObjs[idx] = arrow(cfg); break}
+			case 'cone':          {glowObjs[idx] = cone(cfg); break}
+			case 'cylinder':      {glowObjs[idx] = cylinder(cfg); break}
+			case 'helix':         {glowObjs[idx] = helix(cfg); break}
+			case 'pyramid':       {glowObjs[idx] = pyramid(cfg); break}
+			case 'ring':          {glowObjs[idx] = ring(cfg); break}
+			case 'curve':         {glowObjs[idx] = curve(cfg); break}
+			case 'points':        {glowObjs[idx] = points(cfg); break}
+			case 'vertex':        {glowObjs[idx] = vertex(cfg); break}
+			case 'triangle':      {glowObjs[idx] = triangle(cfg); break}
+			case 'quad':          {glowObjs[idx] = quad(cfg); break}
+			case 'label':         {glowObjs[idx] = label(cfg); break}
+			case 'ellipsoid':     {glowObjs[idx] = sphere(cfg); break}
 			case 'graph':     { // currently graph gives an error for non-fundamental arguments
 				delete cfg.idx
-				glowObjs[idx] = vp_graph(cfg)
+				glowObjs[idx] = graph(cfg)
 				break
 			}
 			case 'gcurve':    { // currently gcurve give an error for non-fundamental arguments
@@ -551,7 +577,7 @@ function handle_cmds(dcmds) {
 				break
 			}
 			case 'gdots':     { // currently gdots give an error for non-fundamental arguments
-				delete cfg.idx
+                delete cfg.idx
 				glowObjs[idx] = gdots(cfg)
 				break
 			}
@@ -723,7 +749,7 @@ function handle_cmds(dcmds) {
 function handle_methods(dmeth) {
     "use strict";
 	//console.log('METHODS')
-	for (var idmeth in dmeth) { // methods; cmd is ['idx':idx, 'attr':method, 'val':val]
+	for (var idmeth=0; idmeth<dmeth.length; idmeth++) { // methods; cmd is ['idx':idx, 'attr':method, 'val':val]
 		var cmd = dmeth[idmeth]
 		var idx = cmd.idx
 		var method = cmd.attr
@@ -788,7 +814,7 @@ function handle_methods(dmeth) {
 function handle_attrs(dattrs) {
     "use strict";
 	//console.log('ATTRS')
-	for (var idattrs in dattrs) { // attributes; cmd is {'idx':idx, 'attr':attr, 'val':val}
+	for (var idattrs=0; idattrs<dattrs.length; idattrs++) { // attributes; cmd is {'idx':idx, 'attr':attr, 'val':val}
 		var cmd = dattrs[idattrs]
 		var idx = cmd.idx
 		var obj = glowObjs[idx]
